import 'package:flutter/material.dart';

mixin _WingsPluginLifecycle {
  Future<Function()> onRunApp(callback());

  Widget onCreateApp(Widget widget);

  Future<void> onPage();
}

abstract class WingsPlugins with _WingsPluginLifecycle {
  List<WingsPlugin> get plugins;

  @override
  onPage() async {
    (plugins ?? []).forEach((e) async {
      await e.onPage();
    });
  }

  @override
  Widget onCreateApp(Widget widget) {
    Widget w = widget;
    (plugins ?? []).forEach((e) {
      w = e.onCreateApp(w);
    });
    return w;
  }

  @override
  onRunApp(callback()) async {
    Function() cb = callback;
    (plugins ?? []).forEach((e) async {
      cb = await e.onRunApp(cb);
    });
    return cb;
  }
}

abstract class WingsPlugin with _WingsPluginLifecycle {
  @override
  Future<Function()> onRunApp(callback) async {
    return callback;
  }

  @override
  onPage() async {}

  @override
  Widget onCreateApp(Widget widget) {
    return widget;
  }
}
